//
//	Includes
//
#include	<MC9S08GB60.h>
#include	"TypeDef.h"
#include	"PortDef.h"

//
//	Prototypes
//
UINT16	ReadModem(UINT8);

//
//	Init port
//
void init_gpio(void)
{
	MODEM_RESET_PULLUP = 0;
	MODEM_CE = 1;					
	MODEM_ATTN = 1;
	MODEM_RTXEN = 0;
	MODEM_RESET = 0;				// Do not initially reset modem 
	MODEM_CE_PORT = 1;
	MODEM_ATTN_PORT = 1;
	MODEM_RTXEN_PORT = 1;
	MODEM_RESET_PORT = 1;
	MODEM_CE = 1;					
	MODEM_ATTN = 1;
	MODEM_RTXEN = 0;
	MODEM_RESET = 0;				// Do not initially reset modem 
}

//
//	Use external clock for fast operation
//
void UseExternalClock(void)
{
	ICGC1 = 0x50;         
	while (!ICGS1_ERCS); // Wait for external reference to be stable. 
	ICGC2_LOLRE = 1;
	ICGC2_MFD = 0x00;		// Mult factor of 4. 
}

//
//	Use internal clock for slow operation
//
void use_mcu_clock(void)
{
	ICGC1_RANGE = 1;	// Range High. 
	ICGC1_OSCSTEN = 1;	// Osc enabled in off mode. 
	ICGC1_CLKS = 0x00;	// Self clocked. 			
}

//
//	Initialize the MCU COP, GPIO, SPI and IRQ
//
void CpuInit(void)
{
UINT16	irq_reg = 0;
UINT8	attn_irq = FALSE, timer_hi, timer_lo;
	
	SOPT = 0x73;	// Turn off the watchdog. 	
	
	// Add a delay to debouce the reset switch on development boards ~200ms 
	TPM1SC = 0x0D;	// Set the Timer module to use BUSCLK as reference with Prescalar at / 32 
	do 
	{
		timer_hi = TPM1CNTH;	// Get the value of the timer register (hi byte) 
		timer_lo = TPM1CNTL;	// Get the value of the timer register (lo byte) 
	}
	while (timer_lo <= 0x80); // Poll for TIMER LO to be greater than 0x80 at 4MHz/32
	TPM1SC = 0x00;	// Return to reset values 

	SPI1C1 = 0x50; // Init SPI 
	SPI1C2 = 0x00;
	SPI1BR = 0x00;

	if(MODEM_RESET_PORT == 0)	// Port direction not set?
	{
		init_gpio();	// Init portdirection
		IRQSC = 0x14; // Turn on the IRQ pin. 
		MODEM_RESET = 1; // Take modem out of reset 
		while(attn_irq == 0)
		{
			if(IRQSC_IRQF == 1)	// Check to see if IRQ is asserted 
			{
				irq_reg = ReadModem(0x24);	// Clear modem interrupts and check for ATTN IRQ from modem
				irq_reg &= 0x400;
				if(irq_reg == 0)
					attn_irq = 0;
				else
					attn_irq = 1;
			}
		}
	}

	IRQSC |= 0x04;	// ACK the pending IRQ interrupt 
	IRQSC = 0x16; // Pin Enable, IE, IRQ CLR, negative edge. 

  TPM1MODH = 0;
  TPM1MODH = 0;
  TPM1MODL = 200;  // 1uS * 200 = 200uS tick

  TPM1CNTH = 0;
  TPM1SC = 0x4b; // Timer divide by 8. (1uS tick for 8MHz .8uS for 10MHz clock).

  SRTISC = SRTISC&~0x07; // Disable wake up timer.
  SPMSC2 = SPMSC2&~0x03; // Enable deep sleep mode stop3.

	RTS_PORT = 1;		//	RTS port dir for output
	RTS_DATA = RTS_ON;	//RTS_OFF;		//	Stop flow for begining
	CTS_PORT = 0;		//	CTS port dir for input

	LED1DIR = 1;	// Set LED1 port as outputs
	LED2DIR = 1;	// Set LED2 port as outputs
	LED3DIR = 1;	// Set LED3 port as outputs
	LED4DIR = 1;	// Set LED4 port as outputs
  
	LED1 = LED_OFF;	// Turn LED1 OFF
	LED2 = LED_OFF;	// Turn LED2 OFF
	LED3 = LED_OFF;	// Turn LED3 OFF
	LED4 = LED_OFF;	// Turn LED4 OFF

	PB0PU = 1; 	// Push button1 directions
	PB1PU = 1;
	PB2PU = 1;
	PB3PU = 1;
}

