#include	<hidef.h>	// for EnableInterrupts macro
#include	<MC9S08GB60.h>	// include peripheral declarations
#include	<stdio.h>
#include	"TypeDef.h"
#include	"PortDef.h"
//
//	Prototypes
//
#include	"Proto.h"

UINT8 SetupData(void);

//
//	Variables
//
UINT8	Master;		// Master or device flag
UINT8	TxSeqNum;	// TX seq number
UINT8	RxSeqNum;	// RX seq number
UINT8	RxLastNum;	// Last RX seq number
UINT8	TickSave;	// Tick value save
volatile UINT8 TickCount;	// Timer tick counter

//
//	Packet structure
//
#define	HEADERSIZE	4		// Protocol headder size
#define	PAYLOADSIZE	56  // Payload size

typedef	struct
{
	UINT8	AckSeq;		// Seq number reply for sender
	UINT8	SeqNum;		// Packet seq number of current packet
	UINT8	Size;		// Payload data size
	UINT8	Time;		// Packet recieved time feedback for sender

	UINT8	data[PAYLOADSIZE];	// Payload data buffer

}	Packet_t;

Packet_t	RxPacket;	// RX packet buffer
Packet_t	TxPacket;	// TX packet buffer

//
//	Timer Interrupt handler
//
interrupt void TimerInterrrupt(void)	// Every 200uS tick
{
	TPM1SC_TOF = 0; 	// Clear Timer int flag(depend on each CPU)
	TickCount++;		// Count up tick timer 	
}

//
//	Main
//
void main(void)
{
UINT8	temp;
//	CPU init
	CpuInit();		// Port init
//	Modem init
	ModemInit();	// Init modem registers
//	Clock setup
	UseExternalClock();	// Set clock sources is modem CLKO (16MHz)
//	SCI init
	SciInit(2);		// SCI init (38400bps)
//	Setup RF parameters
	PowerAdjust(15);	//Set MAX power (0 - 15)
	SetFreq(0);		// Select channel 11 (2405GHz)
//	Initial setting	
	RxSeqNum = 0x80;	// Set dummy to dump first packet
	RxLastNum = 0xff;	// Set dummy for first start up
	TxSeqNum = 1;		// Send packet seq number starts from 1
	TxPacket.Size = 0;	// Set payload is empty
//	Start Main loop
	EnableInterrupts;	// Enable interrupts for operation

	Master = 1;		// 1 = Master mode
//	Master = 0;		// 0 = Device mode (Uncomment when device compile)

	LED3ON;			// Show RTS on
	LED4 = Master? LED_ON: LED_OFF;	// Show Master mode at LED4

#define	LOOPTIME	60	// 200uS * 60 = 12mS main loop
//	Main loop
	for(;;)			// Repeat forever
	{
		TickCount = 0;	// Set timer 0
		temp = SetupData();			// Setup send packet and get length
		SendData((UINT8*)&TxPacket, temp);	// Start transmission
		while(ModemBusy())			// Wait while sending
			LOW_POWER_WHILE;		// Sleep while (wait timer interrput)
		RxSeqNum = 0;				// Clear RX packet seq num (will be set when packet recieved)
		TickSave = 0;				// Clear timing data save
		StartRecieve((UINT8*)&RxPacket, 5000);	// Start receive
		while(ModemBusy() && TickCount < (LOOPTIME - 1))// Wait until packet has come
			LOW_POWER_WHILE;		// Sleep while modem is working
 		if(ModemBusy())				// Still recieving?then it is timeout
 		{
			LED2ON;				// Light error LED
			AbortRecieve();			// Abort receive and set modem idle
 		}
		temp = LOOPTIME;			// Tail time adjust
		if(Master)				// Master mode then adjust main loop time
		{
			if(TickSave == 0)		// No received pakcet?
				temp = LOOPTIME + 5;	// Keep 16ms main loop + a moment to resync
			else if(RxPacket.Time)		// If time data exists in received packet?
				temp = (LOOPTIME * 3 / 2) - RxPacket.Time + 4;// Caluculate time to agjust main loop time
		}
		while(TickCount < temp)			// Keep 12ms main loop time
			LOW_POWER_WHILE;		// Sleep while (waiting timer interrupts)
	}
}

//
//	Setup Transmit data
//
UINT8 SetupData(void)
{
UINT8	index, getc;
//	RTS flow control
	if(RTS_DATA == RTS_OFF)			// Now RTS is off?
	{
		if(GetRxFreeSize() > 0x300)	// RX buffer has enough space?
		{
			RTS_DATA = RTS_ON;	// RTS ON
			LED3ON;			// Show RTS on
		}
	}
//	Setup send data buffer
	index = TxPacket.Size;			// Get data size of current send packet
	if(index)				// Still exists?
	{
		TxPacket.AckSeq = RxSeqNum;	// Set Ack seq num
		return TxPacket.Size + HEADERSIZE;// Wait until sent successfully (send same payload again)
	}

	while(CheckSciData())			// Repeat while SCI data exists
	{
		getc = GetSciData();		// Get char
		TxPacket.data[index] = getc;	// Store char to packet buffer
		TxPacket.Size = ++index;	// Add payload length
		if(index == PAYLOADSIZE)	// Buffer full?
			break;			// Exit packet data set loop
	}

	TxPacket.AckSeq = RxSeqNum;		// Set Ack seq num of received packet
	if(++TxSeqNum == 0)			// Increment send packet seq number for new packet
		TxSeqNum++;			// If it's 0 then force set to 1
	TxPacket.SeqNum = TxSeqNum;		// Set send packet seq num
	TxPacket.Time = TickSave;		// Set received packet arrival time to sync
	return index + HEADERSIZE;		// Return with new packet + header length				
}

//
//	Data receive indication callback
//
void DataIndication(UINT8 stat)
{
UINT8	i;
	if(stat == SUCCESS)			// This receive was successul?
	{
		TickSave = TickCount - (RxPacket.Size >> 2);	// Save packet arrival time (adjusted by data len)
		LED1XOR;			// Flip packet received LED
		LED2OFF;			// Clear error LED
//	Check my packet has received successfuly?
		if(RxPacket.AckSeq == TxSeqNum)	// Ack seq matches sent seq num?
			TxPacket.Size = 0;	// Clear size for next packet send buffering
//	Flow control
		if(CTS_DATA == CTS_OFF || GetTxFreeSize() < 0x100)	// CTS is off or TX buffer has no space?
		{
			RxSeqNum = 0;		// Force retry if TX buffer is full
			return;			// Return (expect to be resent by sender) 
		}
//	Check seq number
		RxSeqNum = RxPacket.SeqNum;	// Save received packet seq number
		if(RxSeqNum == RxLastNum)	// Already ack processed?
			return;			// Return (it was duplicate packet sending by sender)
		RxLastNum = RxSeqNum;		// Save processed packet seq num
//	Send received data to SCI data queue
		for(i = 0; i < RxPacket.Size; i++)	// Send all recieved data to 232C
			PutSciData(RxPacket.data[i]);	// Put data to SCI send queue
		SciTxStart();			// Start SCI transmission
	}
	else
		LED2ON;				// Show error
}
