#include "TypeDef.h"

#ifndef _MODEM_REGS_H
#define	_MODEM_REGS_H

// modem soft reset 
#define	RESET                   0x00

// Packet RAM 
#define	RX_PKT                  0x01    // RX Packet RAM 
#define	RX_PKT_LEN              0x2D    // RX Packet RAM Length [6:0] 
#define	TX_PKT                  0x02    // TX Packet RAM 
#define	TX_PKT_LEN              0x03    // TX Packet RAM Length 
#define	TX_PKT_LEN_MASK         0x007F  // TX Packet RAM Length Mask 

// IRQ Status Register 
#define	IRQ_MASK                0x05
#define	STATUS_ADDR             0x24
#define	RESIND_ADDR             0x25
#define	TIMER1_IRQMASK_BIT      0x0001

// Mask and mode 
#define	MODE_ADDR               0x06     
#define	MODE2_ADDR              0x07     
#define	LO1_COURSE_TUNE         0x8000

// Main Timer 
#define	TIMER_PRESCALE          0x09
#define	TIMESTAMP_HI_ADDR       0x26
#define	TIMESTAMP_LO_ADDR       0x27
#define	TIMESTAMP_HI_MASK       0x00FF

// Frequency 
#define	XTAL_ADJ_ADDR           0x0A
#define	CLKS_ADDR               0x0A
#define	LO1_IDIV_ADDR           0x0F
#define	LO1_NUM_ADDR            0x10
#define	PRESCALE_ADDR           0x09

// Timer comparators 
#define	T1_HI_ADDR              0x1B    
#define	T1_LO_ADDR              0x1C
#define	T2_HI_ADDR              0x1D    
#define	T2_LO_ADDR              0x1E

// CCA 
#define	CCA_THRESHOLD           0x04
#define	CCA_RESULT_ADDR         0x2D
#define	FEGAIN_ADDR             0x04

// TX 
#define	PA_ADJUST_ADDR          0x12

// GPIO 
#define	GPIO_CONFIG             0x0B
#define	GPIO_DATA               0x0C
#define	GPIO_DATA_MASK          0x003F

// Version 
#define	VERSION_REG             0x2C
#define	VERSION_MASK            0x1C00

//	Test registers
#define	BER_REG                 0x30
#define	BER_MASK                0x8000
#define	PSM_REG                 0x31
#define	PSM_MASK                0x0008
#define	PAEN_REG                0x08
#define	PAEN_MASK               0x8000

#endif // _MODEM_REGS_H 


//	Modes defined for the transciever
#define IDLE_MODE								0x00
#define RX_MODE									0x02
#define TX_MODE									0x03
#define CCA_MODE								0x01
#define RX_MODE_WTO							0x80
#define HIBERNATE_MODE					0x81
#define DOZE_MODE								0x82
#define IDLE_MODE_DOZE					0x83
#define IDLE_MODE_ATTN					0x84
#define TIMEOUT									0x85
#define	SYSTEM_RESET_MODE				0x86
#define MODEM_RESET_MODE				0x87    // Not implemented
#define MODEM_CONFIG_MODE				0x88
#define RESET_DELAY							0x89    // Set to 100ms

#define MAX_POWER 100	// Numbers chosen arbitrarily but > 16
#define NOMINAL_POWER 0x0B
#define MIN_POWER 50	// Numbers chosen arbitrarily but > 16

// Modem receive status
#define SUCCESS		0
#define	CRCERROR	1
#define OVERFLOW	2

#define INITIAL_VALUE 0x0

//	Status of DataMode
#define	SET_IDLE		0
#define	SET_TXDATA	1
#define	SET_RXDATA	2

// Silent port settings. 
#define	MODEM_CE					PTED_PTED2				
#define	MODEM_CE_PORT			PTEDD_PTEDD2				
#define	MODEM_ATTN				PTDD_PTDD0
#define	MODEM_ATTN_PORT		PTDDD_PTDDD0
#define	MODEM_RTXEN				PTDD_PTDD1
#define	MODEM_RTXEN_PORT	PTDDD_PTDDD1
#define	MODEM_RESET				PTDD_PTDD3
#define	MODEM_RESET_PORT	PTDDD_PTDDD3
#define	MODEM_RESET_PULLUP 	PTDPE_PTDPE3
#define	MODEM_IRQ_SOURCE	IRQSC
#define	MODEM_IRQ_IE_BIT	IRQSC_IRQIE

#define	LED1					PTAD_PTAD0
#define	LED1DIR				PTADD_PTADD0
#define	LED2					PTAD_PTAD1
#define	LED2DIR				PTADD_PTADD1
#define	LED3					PTAD_PTAD2
#define	LED3DIR				PTADD_PTADD2
#define	LED4					PTAD_PTAD3
#define	LED4DIR				PTADD_PTADD3
#define	LEDPORT 			PTAD
#define	PB0						PTBD_PTBD4
#define	PB0PU					PTBPE_PTBPE4
#define	PB0DIR 				PTBDD_PTBDD4
#define	PB1 					PTBD_PTBD5
#define	PB1PU 				PTBPE_PTBPE5
#define	PB1DIR 				PTBDD_PTBDD5
#define	PB2 					PTBD_PTBD6
#define	PB2PU 				PTBPE_PTBPE6
#define	PB2DIR 				PTBDD_PTBDD6
#define	PB3 					PTBD_PTBD7
#define	PB3PU 				PTBPE_PTBPE7
#define	PB3DIR 				PTBDD_PTBDD7

#define	PUSH_BUTTON1 PTBD_PTBD4
#define	PUSH_BUTTON2 PTBD_PTBD5
#define	PUSH_BUTTON3 PTBD_PTBD6
#define	PUSH_BUTTON4 PTBD_PTBD7
	
#define	LED_OFF 1
#define	LED_ON 0
#define	PRESSED	0

#define	LED1ON	LED1 = LED_ON
#define	LED1OFF	LED1 = LED_OFF
#define	LED1XOR	LED1 ^= 1
#define	LED2ON	LED2 = LED_ON
#define	LED2OFF	LED2 = LED_OFF
#define	LED2XOR	LED2 ^= 1
#define	LED3ON	LED3 = LED_ON
#define	LED3OFF	LED3 = LED_OFF
#define	LED3XOR	LED3 ^= 1
#define	LED4ON	LED4 = LED_ON
#define	LED4OFF	LED4 = LED_OFF
#define	LED4XOR	LED4 ^= 1

#define	DEB1ON  PTCD_PTCD0 = 1
#define	DEB1OFF	PTCD_PTCD0 = 0
#define	DEB1XOR	PTCD_PTCD0 ^= 1
#define	DEB2ON  PTCD_PTCD1 = 1
#define	DEB2OFF PTCD_PTCD1 = 0
#define	DEB2XOR PTCD_PTCD1 ^= 1

#define	RTS_PORT	PTDDD_PTDDD2	// RTS port direction for output
#define	RTS_DATA	PTDD_PTDD2	// RTS port direction for output
#define	CTS_PORT	PTCDD_PTCDD3	// CTS port direction for output
#define	CTS_DATA	PTCD_PTCD3	// CTS data

#define	RTS_ON		0		// Invert
#define	RTS_OFF		1		// Invert
#define	CTS_ON		0		// Invert
#define	CTS_OFF		1		// Invert
	
#define	SCIBDH   SCI1BDH
#define	SCIBDL   SCI1BDL
#define	SCIC1    SCI1C1 
#define	SCIC2    SCI1C2 
#define	SCIC2_TIE	SCI1C2_TIE
#define	SCIS1    SCI1S1 
#define	SCIS2    SCI1S2 
#define	SCIC3    SCI1C3 
#define	SCID     SCI1D

#define	SCIS1_TDRE SCI1S1_TDRE
#define	SCIS1_TC	 SCI1S1_TC

#define	KBDWAIT _asm stop

#define	MODEM_CLKO_FREQ	0x01 // Sets the modem CLKO frequency 
#define	LOW_POWER_WHILE		asm("wait")

