//
//	Includes
//
#include	<MC9S08GT60.h>
#include	"TypeDef.h"
#include	"PortDef.h"

//	initialisation constants
#define baud57600 0x09  // 8.66
#define	baud38400	0x0D	// Value = 500000 / baudrate
#define	baud19200	0x1a	// 19200bps
#define	baud9600	0x34	// 9600bps

//	Variables
UINT8	RxRingBuf[1024];	// Ring buffer
volatile UINT16	RxRingOut, RxRingIn;	// Ring pointer
#define	RX_MASK	0x3ff
#define	RX_FULL		0x300

UINT8	TxRingBuf[1024];	// Ring buffer
volatile UINT16	TxRingOut, TxRingIn;	// Ring pointer
#define	TX_MASK	0x3ff

//
//	Init SCI registers
//
void	SciInit(UINT8 baud)
{
	SCIBDH = 0x00;		// Set baudrate high byte
	switch(baud)
	{
	case 0:	// 9600bps
		SCIBDL = baud9600;	// Set baudrate low byte
		break;
	case 1:	// 19200bps
		SCIBDL = baud19200;	// Set baudrate low byte
		break;
	case 2:	// 38400bps
		SCIBDL = baud38400;	// Set baudrate low byte
		break;
	default:// 57600bps
		SCIBDL = baud57600;	// Set baudrate low byte
		break;
	}
	SCIC2 = 0b10101100;	// RX interrupt enable, TX & RX enable
	SCIC3 = 0b00001110;	// Enable OR, NE, FE error interrupt
	RxRingOut = RxRingIn = 0;	// Clear ring buffer pointer
	TxRingOut = TxRingIn = 0;	// Clear ring buffer pointer
}

//
//	SCI RX interrupt handler
//
interrupt void SciRxInterrupt()
{	
UINT8	SciStat, SciData;
	SciStat = SCIS1;
	SciData = SCID;	// Read data from SCI
//	It's good data then store to buffer
	if(RxRingOut != ((RxRingIn + 1) & RX_MASK))	// Have space to put?
	{
		RxRingBuf[RxRingIn] = SciData;
		RxRingIn = ((RxRingIn + 1) & RX_MASK);
	}
//	Flow control (Stop)
	if(RTS_DATA == RTS_ON																// Now RTS is on?
	 && (((RxRingIn - RxRingOut) & RX_MASK) > RX_FULL))	// Exceed 3/4?
	{
		RTS_DATA = RTS_OFF;																// Make RTS off
		LED3OFF;																					// Show flow
	}
}

//
//	SCI TX interrupt handler
//
interrupt void SciTxInterrupt()
{	
UINT8	SciStat;
	SciStat = SCIS1;
	if(!(SciStat & 0x80))
		return;
	if(TxRingIn == TxRingOut)
	{
		SCIC2_TIE = 0;
		return;
	}
	SCID = TxRingBuf[TxRingOut];
	TxRingOut = ((TxRingOut + 1) & TX_MASK);
}

//
//	SCI Send data by timer
//
void SciTxStart()
{
	SCIC2_TIE = 1;
}

//
//	SCI Error interrupt handler
//
interrupt void SciErrInterrupt()
{	
UINT8	SciStat;
	SciStat = SCIS1;
	SciStat = SCID; // Clear error status
}

//
//	Get data from RX ring buffer
//
UINT8 GetSciData(void)
{
UINT8 getc;
	if(RxRingIn == RxRingOut)	// Check SCI data has come?
		return 0;		//
	getc = RxRingBuf[RxRingOut];	// Get char
	RxRingOut = ((RxRingOut + 1) & RX_MASK);	// Adjust pointer
	return getc;
}

//
//	Test RX ring buffer has data
//
UINT8 CheckSciData(void)
{
	return (RxRingIn != RxRingOut);	// Check SCI data has come?
}

//
//	Put data to TX ring buffer
//
UINT8 PutSciData(UINT8 ch)
{
	if(TxRingOut == ((TxRingIn + 1) & TX_MASK))	// Have space to put?
		return 0;	// Fail return(ring buffer is full)
	TxRingBuf[TxRingIn] = ch;	// Put data to 232C send queue
	TxRingIn = (TxRingIn + 1) & TX_MASK;	// Increment pointer
	return 1;
}

//
//	Get TX ring buffer free size
//
UINT16	GetTxFreeSize(void)
{
	return 1024 - ((TxRingIn - TxRingOut) & TX_MASK);
}

//
//	Get RX ring buffer free size
//
UINT16	GetRxFreeSize(void)
{
	return 1024 - ((RxRingIn - RxRingOut) & RX_MASK);
}

//
//	Put one char to SCI
//
void SciPutc(UINT8 c)
{
	while(!SCIS1_TDRE); // Check TX data buffer empty
	while(!SCIS1_TC);	 // wait for Tx complete
	SCID = c;	 // load data to SCI2 register
}

//
//	Put string to SCI
//
void SciPuts(UINT8 *s)
{
	while(*s)
		SciPutc(*s++);
}
